{ TEmiStringGrid component version 1.1

  Author : Eric MINET - eric.minet@libertysurf.fr

  Status : Freeware with sources

  Target : Delphi 1.0 to Delphi 5.0, maybe C++ Builder 1.0-4.0

  Derived from TStringGrid. Based on idea from Robert Vivrette.

  Added features :

  published properties :
  - ColTitles : set text for each cell of the first row (column titles)
  - RowTitles : set text for each cell of the first column (row titles)
  - DefaultCellsAlignment : defines the default text alignment for all
    cells of the grid at design-time.
  - DefaultMultiLineCells : defines the default text multi-line capability
    at design-time.

  Events :
  - OnGetCellColor : allows Brush, Font, alignment and multi-line
    capability to be set at run-time for each cell of the grid. }
    
unit emiSGrid;

{Component palette bitmap}
{$IFDEF WIN32}
   {$R emiSGrid.d32}
{$ELSE}
   {$R emiSGrid.d16}
{$ENDIF}

interface

uses WinProcs, WinTypes, Classes, Graphics, Grids, Messages, Dialogs, SysUtils;

{Component palette tab name for register method}
const ThePalettePage = 'Add-Ons';

type
  { Cell alignment values }
  TCellAlignment = (caTopLeft, caTopCenter, caTopRight, caMiddleLeft,
    caCenter, caMiddleRight, caBottomLeft, caBottomCenter, caBottomRight);
  { OnGetCellColor event declaration }
  TColorEvent = procedure (Sender: TObject;                   {Sender : The grid}
                           ARow, ACol: Longint;               {Cell coordinates}
                           AState: TGridDrawState;            {Cell state}
                           var Alignment  : TCellAlignment;   {Cell text alignment}
                           var MultiLine  : boolean;          {Multi-line capability}
                           ABrush: TBrush;                    {Canvas Brush}
                           AFont: TFont                       {Canvas Font}
                ) of object;
  { Component declaration}
  TEmiStringGrid = class(TStringGrid)
  private
    FonGetCellColor: TColorEvent;
    FRowTitles : TStrings;
    FColTitles : TStrings;
    FDefaultCellsAlignment  : TCellALignment;
    FDefaultMultiLineCells  : boolean;
    procedure SetRowTitles(Value : TStrings);
    procedure SetColTitles(Value : TStrings);
    procedure SetDefaultCellsAlignment(Value : TCellALignment);
    procedure SetDefaultMultiLineCells(Value : boolean);
  protected
    procedure DrawCell(ACol, ARow: Longint; ARect: TRect;
              AState: TGridDrawState); override;
    procedure Loaded; override;
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
  published
    property OnGetCellColor: TColorEvent read  FOnGetCellColor
                                         write FOnGetCellColor;
    property RowTitles : TStrings read FRowTitles write SetRowTitles;
    property ColTitles : TStrings read FColTitles write SetColTitles;
    property DefaultCellsAlignment : TCellALignment
             read FDefaultCellsAlignment write SetDefaultCellsAlignment;
    property DefaultMultiLineCells : boolean
             read FDefaultMultiLineCells write SetDefaultMultiLineCells;
  end;

procedure Register;

implementation

procedure TEmiStringGrid.SetRowTitles(Value : TStrings);
begin
  FRowTitles.Assign(Value);
  if ColCount>0 then
  begin
    Cols[0].Clear;
    Cols[0].Assign(FRowTitles);
  end;
  Invalidate;
end;

procedure TEmiStringGrid.SetColTitles(Value : TStrings);
begin
  FColTitles.Assign(Value);
  if RowCount>0 then
  begin
    Rows[0].Clear;
    Rows[0].Assign(FColTitles);
  end;
  Invalidate;
end;

procedure TEmiStringGrid.SetDefaultCellsAlignment(Value : TCellALignment);
begin
  if Value<>FDefaultCellsAlignment then
  begin
    FDefaultCellsAlignment:=Value;
    {Multi-line allowed only for top-aligned text}
    FDefaultMultiLineCells:=FDefaultMultiLineCells
      and (Value in [caTopLeft, caTopCenter, caTopRight]);
    Invalidate;
  end;
end;

procedure TEmiStringGrid.SetDefaultMultiLineCells(Value : boolean);
begin
  if Value<>FDefaultMultiLineCells then
  begin
    FDefaultMultiLineCells:=Value;
    if Value then
      {Text alignment must be top-something if multi-line is selected}
      case FDefaultCellsAlignment of
        caMiddleLeft,caBottomLeft : FDefaultCellsAlignment:=caTopLeft;
        caCenter, caBottomCenter  : FDefaultCellsAlignment:=caTopCenter;
        caMiddleRight, caBottomRight : FDefaultCellsAlignment:=caTopRight;
      end;
    Invalidate;
  end;
end;

procedure TEmiStringGrid.Loaded;
begin
  inherited Loaded;
  if ColCount>0 then
  begin
    Cols[0].Clear;
    Cols[0].Assign(FRowTitles);
  end;
  if RowCount>0 then
  begin
    Rows[0].Clear;
    Rows[0].Assign(FColTitles);
  end;
end;

constructor TEmiStringGrid.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FRowTitles:=TStringList.Create;
  FColTitles:=TStringList.Create;
  FDefaultCellsAlignment:=caTopLeft;
  FDefaultMultiLineCells:=false;
end;

destructor TEmiStringGrid.Destroy;
begin
  FRowTitles.Free;
  FColTitles.Free;
  inherited Destroy;
end;

procedure TEmiStringGrid.DrawCell(ACol, ARow: Longint; ARect: TRect;
          AState: TGridDrawState);
var AAlignment   : TCellAlignment;
    AMultiLine   : boolean;
    TheRect      : TRect;
    {$IFDEF WIN32}
    AFormat      : Longint;
    S            : string;
    {$ELSE}
    AFormat      : word;
    P            : PChar;
    {$ENDIF}
begin
  {Default alignment}
  AAlignment:=FDefaultCellsAlignment;
  {Default multi-line capability}
  AMultiLine:=FDefaultMultiLineCells;
  {Call the event if assigned}
  if Assigned(FOnGetCellColor) then 
    FOnGetCellColor(Self, ARow, ACol, AState, AAlignment,
      AMultiLine, Canvas.Brush, Canvas.Font);
  {Ensure text is top-aligned if multi-line is selected}
  AMultiLine:=AMultiLine and (AAlignment in [caTopLeft, caTopCenter, caTopRight]);
  if DefaultDrawing
  then
    begin
      if AMultiLine
      then AFormat:=DT_NOPREFIX+DT_WORDBREAK
      else AFormat:=DT_NOPREFIX+DT_SINGLELINE;
      case AAlignment of
        caTopCenter    : AFormat:=AFormat+DT_TOP+DT_CENTER;
        caTopRight     : AFormat:=AFormat+DT_TOP+DT_RIGHT;
        caMiddleLeft   : AFormat:=AFormat+DT_VCENTER+DT_LEFT;
        caCenter       : AFormat:=AFormat+DT_VCENTER+DT_CENTER;
        caMiddleRight  : AFormat:=AFormat+DT_VCENTER+DT_RIGHT;
        caBottomLeft   : AFormat:=AFormat+DT_BOTTOM+DT_LEFT;
        caBottomCenter : AFormat:=AFormat+DT_BOTTOM+DT_CENTER;
        caBottomRight  : AFormat:=AFormat+DT_BOTTOM+DT_RIGHT;
        else AFormat:=AFormat+DT_TOP+DT_LEFT;
      end;
      {Offsets adjustments}
      TheRect:=ARect;
      Canvas.FillRect(TheRect);
      TheRect.Top:=TheRect.Top+2;
      TheRect.Bottom:=TheRect.Bottom-2;
      TheRect.Left:=TheRect.Left+2;
      TheRect.Right:=TheRect.Right-2;
      {$IFDEF WIN32}
      S := Cells[ACol, ARow];
      DrawText(Canvas.Handle, PChar(S), Length(S), TheRect, AFormat);
      {$ELSE}
      P:=StrAlloc(Length(Cells[ACol, ARow])+1);
      StrPCopy(P,Cells[ACol, ARow]);
      DrawText(Canvas.Handle, P, Length(Cells[ACol, ARow]), TheRect, AFormat);
      StrDispose(P);
      {$ENDIF}
    end
  else inherited DrawCell(ACol, ARow, ARect, AState);
end;

procedure Register;
begin
  RegisterComponents(ThePalettePage, [TEmiStringGrid]);
end;

end.
